package com.norconex.blog.facebook.crawler;

import java.io.IOException;
import java.io.Reader;
import java.io.UnsupportedEncodingException;
import java.io.Writer;
import java.net.URLEncoder;

import javax.xml.stream.XMLStreamException;

import org.apache.commons.configuration.XMLConfiguration;
import org.apache.commons.lang3.CharEncoding;
import org.apache.http.client.HttpClient;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

import com.norconex.collector.core.CollectorException;
import com.norconex.collector.http.doc.HttpDocument;
import com.norconex.collector.http.fetch.HttpFetchResponse;
import com.norconex.collector.http.fetch.impl.GenericDocumentFetcher;
import com.norconex.commons.lang.config.ConfigurationUtil;
import com.norconex.commons.lang.url.URLStreamer;
import com.norconex.commons.lang.xml.EnhancedXMLStreamWriter;

/**
 * Fetches and retrieves Facebook posts.
 * <p>
 * XML configuration usage:
 * </p>
 * <pre>
 *  &lt;httpDocumentFetcher  
 *      class="com.norconex.blog.facebook.crawler.FacebookDocumentFetcher"&gt;
 *      &lt;appId&gt;(facebook client id)&lt;/appId&gt;
 *      &lt;appSecret&gt;(facebook client secret)&lt;/appSecret&gt;
 *  &lt;/httpDocumentFetcher&gt;
 * </pre>
 * @author Pascal Essiembre
 */
public class FacebookDocumentFetcher extends GenericDocumentFetcher {

    private static final Logger LOG = 
            LogManager.getLogger(FacebookDocumentFetcher.class);
    
    // This is the URL to use for dynamically obtaining a fresh access token
    private static final String GRAPH_API_OAUTH_URL =
            "https://graph.facebook.com/oauth/access_token?client_id=%s"
          + "&client_secret=%s&grant_type=client_credentials";
        
    private String accessToken;
    private String appId;
    private String appSecret;

    @Override
    public HttpFetchResponse fetchDocument(
            HttpClient httpClient, final HttpDocument doc) {
        
        // Make sure we have a Facebook Graph API access token
        ensureAccessToken(httpClient);
        
        // Append the access token to the URL if it does not already exists
        String originalReference = doc.getReference();
        if (!originalReference.contains("&access_token=")) {
            String oauthReference = appendAccessToken(originalReference);
            doc.setReference(oauthReference);
        }

        // Let the parent class make the Facebook Graph API call
        return super.fetchDocument(httpClient, doc);
    }
    
    private String appendAccessToken(String reference) {
        String sep = "&";
        if (!reference.contains("?")) {
            sep = "?";
        }
        return reference + sep + "access_token=" + accessToken;
    }
    
    private synchronized void ensureAccessToken(HttpClient httpClient) {
        if (accessToken != null) {
            return;
        }
        String url = String.format(GRAPH_API_OAUTH_URL, appId, appSecret);
        LOG.debug("Extracting code using: " + url);
        String response = URLStreamer.streamToString(url);
        LOG.debug("Response: " + response);
        if (response.contains("access_token")) {
            String token = response.replaceFirst(
                    "\\{\"access_token\"\\:\"(.*?)\".*", "$1");
            try {
                accessToken = URLEncoder.encode(token, CharEncoding.UTF_8);
            } catch (UnsupportedEncodingException e) {
                throw new CollectorException(e);
            }
        } else {
            throw new CollectorException(
                    "Could not obtain access token. Response was: " + response);
        }
    }

    public String getAppId() {
        return appId;
    }
    public void setAppId(String clientId) {
        this.appId = clientId;
    }
    public String getAppSecret() {
        return appSecret;
    }
    public void setAppSecret(String clientSecret) {
        this.appSecret = clientSecret;
    }

    @Override
    public void loadFromXML(Reader in) {
        XMLConfiguration xml = ConfigurationUtil.newXMLConfiguration(in);
        setAppId(xml.getString("appId"));
        setAppSecret(xml.getString("appSecret"));
    }
    @Override
    public void saveToXML(Writer out) throws IOException {
        try {
            EnhancedXMLStreamWriter writer = new EnhancedXMLStreamWriter(out);
            writer.writeStartElement("httpDocumentFetcher");
            writer.writeAttribute("class", getClass().getCanonicalName());
            writer.writeStartElement("appId");
            if (appId != null) {
                writer.writeCharacters(appId);
            }
            writer.writeEndElement();
            writer.writeStartElement("appSecret");
            if (appSecret != null) {
                writer.writeCharacters(appSecret);
            }
            writer.writeEndElement();
            writer.writeEndElement();
            writer.flush();
            writer.close();
        } catch (XMLStreamException e) {
            throw new IOException("Cannot save as XML.", e);
        }        
    }
}
