package com.norconex.blog.facebook.crawler;

import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.xml.stream.XMLStreamException;

import org.apache.commons.configuration.XMLConfiguration;
import org.apache.commons.lang3.StringUtils;

import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import com.google.gson.stream.JsonReader;
import com.norconex.commons.lang.io.CachedStreamFactory;
import com.norconex.commons.lang.xml.EnhancedXMLStreamWriter;
import com.norconex.importer.doc.ImporterDocument;
import com.norconex.importer.doc.ImporterMetadata;
import com.norconex.importer.handler.ImporterHandlerException;
import com.norconex.importer.handler.splitter.AbstractDocumentSplitter;
import com.norconex.importer.handler.splitter.SplittableDocument;

public class FacebookDocumentSplitter extends AbstractDocumentSplitter {

    public FacebookDocumentSplitter() {
    }

    protected List<ImporterDocument> splitApplicableDocument(
            SplittableDocument parentDoc, OutputStream output,
            CachedStreamFactory streamFactory, boolean parsed)
            throws ImporterHandlerException {

        // First, we make sure only Facebook posts are split here but returning
        // null on non-post references.
        if (!FacebookUtils.isPosts(parentDoc.getReference())) {
            return null;
        }

        List<ImporterDocument> postDocs = new ArrayList<>();
        ImporterMetadata parentMeta = parentDoc.getMetadata();

        JsonReader jsonReader = new JsonReader(parentDoc.getReader());
        jsonReader.setLenient(true);

        JsonObject json = null;
        try {
            json = (JsonObject) new JsonParser().parse(jsonReader);
        } catch (ClassCastException e) {
            throw new ImporterHandlerException("Cannot parse JSON input.", e);
        }

        // Each top-level "data" element is a single document/post.
        JsonArray postsData = json.get("data").getAsJsonArray();
        Iterator<JsonElement> it = postsData.iterator();
        while (it.hasNext()) {
            JsonObject postData = (JsonObject) it.next();
            try {
                ImporterDocument doc = createImportDocument(
                        postData, parentDoc.getReference(), parentMeta,
                        streamFactory);
                if (doc != null) {
                    postDocs.add(doc);
                }
            } catch (IOException e) {
                throw new ImporterHandlerException(e);
            }
        }
        return postDocs;
    }

    protected ImporterDocument createImportDocument(
            JsonObject json, String parentRef, ImporterMetadata parentMeta,
            CachedStreamFactory streamFactory) 
                    throws IOException {

        if (json == null) {
            return null;
        }

        // METADADA
        ImporterMetadata childMeta = new ImporterMetadata();

        // Optionally assign values from master JSON document
//        childMeta.load(parentMeta);
        
        // Parse and assign any values you need for a child
        String id = getString(json, "id");
        childMeta.setString("id", id);
        childMeta.setString("description", getString(json, "description"));
        childMeta.setString("link", getString(json, "link"));
        childMeta.setString("picture", getString(json, "picture"));
        childMeta.setString("type", getString(json, "type"));
        childMeta.setString("created_time", getString(json, "created_time"));

        JsonObject from = (JsonObject) json.get("from");
        String fromName = getString(from, "name");
        childMeta.setString("from_name", fromName);
        childMeta.setString("from_id", getString(from, "id"));

        // Consider "message" as the document "content".
        String content = getString(json, "message");
        if (StringUtils.isBlank(content)) {
            return null;
        }
        
        // Create a unique reference for this child element.  Let's make it
        // the URL to access this post in a browser.
        // e.g. https://www.facebook.com/Disney/posts/10152684762485954
        String ref = "https://www.facebook.com/" + fromName + "/posts/" 
                + StringUtils.substringAfter(id, "_");
        childMeta.setString(ImporterMetadata.DOC_REFERENCE, ref);

        // Set parent reference if you need it, and optionally remove 
        // the access token from it to keep it clean
//        String parentReference = parentRef.replaceFirst(
//                "(.*)&access_token=.*", "$1");
//        childMeta.setEmbeddedParentReference(parentReference);
//        childMeta.setEmbeddedReference(ref);

        // We gathered enough data for a single doc, return it
        return new ImporterDocument(
                ref, streamFactory.newInputStream(content), childMeta);
    }

    // Convenience method for getting a string for a JSON object.
    private String getString(JsonObject jsonObject, String memberName) {
        if (jsonObject == null) {
            return null;
        }
        JsonElement element = jsonObject.get(memberName);
        if (element != null) {
            return element.getAsString();
        }
        return null;
    }
    
    @Override
    protected void loadHandlerFromXML(XMLConfiguration xml) throws IOException {
        // nothing extra to load
    }

    @Override
    protected void saveHandlerToXML(EnhancedXMLStreamWriter writer)
            throws XMLStreamException {
        // nothing extra to save
    }
}
